import {ComponentFixture, fakeAsync, TestBed, tick} from '@angular/core/testing';
import {of} from 'rxjs';
import {FormsModule, NgForm} from '@angular/forms';
import {AppModule} from '../../app.module';
import {map} from 'rxjs/operators';
import {By} from '@angular/platform-browser';
import {DataCardService} from '../../api/data-card.service';
import {ObservationService} from '../../api/observation.service';
import {MeasurementsModalComponent} from './measurements-modal.component';
import {CarePlanService} from '../../api/care-plan.service';
import {
  Activity,
  ObservationTable,
  ObservationTableFetuses,
  ScanningObservationTable,
  ScanningObservationTableFetuses
} from '@nspop/gm-web-facade-api';
import {SimpleChange, SimpleChanges} from '@angular/core';

function getActivity(): Activity {
  return {title: 'some activity', id: '1234abcd'} as Activity;
}

describe('MeasurementsModalComponent', () => {
  let component: MeasurementsModalComponent;
  let fixture: ComponentFixture<MeasurementsModalComponent>;
  let dataCardServiceStub: DataCardService;
  let carePlanServiceStub: CarePlanService;
  let observationService: ObservationService;

  beforeEach(async () => {
    const dummyDataCardResponse = {
      expectedDateOfDeliverySection: {},
      prenatalRiskAssessmentSection: {
        finalRiskAssessment: {
          multiplicityRiskAssessment: {}
        },
      },
      overallAssessmentSection: {prenatalCareAssessment: {numberOfFetuses: 2}}
    };
    dataCardServiceStub = {
      isDataCardClosed: of(false),
      selectedDataCard$: of(dummyDataCardResponse),
      refresh: () => of(),
      update: () => of(dummyDataCardResponse),
    } as any;

    carePlanServiceStub = {
      getActivityById: (activityId: string) => undefined,
    } as any;

    observationService = {
      createUltrasoundObservations: () => of(),
      createRegularObservations: () => of(),
      ultrasoundObservationTable$: of({mother: {rows: [{activityId: '1234-abcd', id: '1', flow: 'afvigende'}]}} as ScanningObservationTable),
      clinicalObservationTable$:  of({mother: {rows: [{activityId: '1234-abcd', id: '1'}]}} as ObservationTable)
    } as any;

    await TestBed.configureTestingModule({
      declarations: [ MeasurementsModalComponent ],
      imports: [FormsModule, AppModule],
      providers: [
        {provide: DataCardService, useValue: dataCardServiceStub},
        {provide: ObservationService, useValue: observationService},
        {provide: NgForm, useValue: new NgForm([], [])},
      ]
    })
    .compileComponents();
  });

  beforeEach(() => {
    fixture = TestBed.createComponent(MeasurementsModalComponent);
    component = fixture.componentInstance;
    component.observationDataUltrasound = {};
    component.observationDataRegular = {};
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  it('fetches datacard and resets observationData on init', fakeAsync(() => {
    let accessed = false;
    dataCardServiceStub.selectedDataCard$ = of({multiplePregnancySection: {numberOfFetuses: 3}} as any).pipe(
      map(x => {accessed = true; return x; })
    );

    fixture = TestBed.createComponent(MeasurementsModalComponent);
    component = fixture.componentInstance;
    fixture.detectChanges();

    component.isOpen = true;
    component.ngOnChanges({isOpen: new SimpleChange(false, true, true)});
    fixture.detectChanges();
    component.ngOnInit();
    tick();
    expect(accessed).toBeTrue();
    expect(component.observationDataRegular?.fetuses?.length).toBe(3);
  }));

  it('allows save if activity was picked', fakeAsync(() => {
    component.isOpen = true;
    component.activity = getActivity();
    component.ngOnChanges({isOpen: new SimpleChange(false, true, true)});
    component.observationType = 'regular';
    const spy = spyOn(observationService, 'createRegularObservations').and.callThrough();
    fixture.detectChanges();
    component.doSave();
    tick();
    expect(spy).toHaveBeenCalled();
  }));

  it('does not allow save if no activity was picked', fakeAsync(() => {
    component.activity = undefined;
    component.isOpen = true;
    component.ngOnChanges({isOpen: new SimpleChange(false, true, true)});
    component.observationType = 'regular';
    const spy = spyOn(observationService, 'createRegularObservations').and.callThrough();
    component.observationDataRegular = {} as any;
    fixture.detectChanges();
    component.doSave();
    tick();
    expect(spy).not.toHaveBeenCalled();
  }));

  it('should set measurementDate when activity is saved', fakeAsync(() => {
    component.isOpen = true;
    component.activity = getActivity();
    component.observationType = 'regular';
    component.ngOnChanges({isOpen: new SimpleChange(false, true, true)});
    component.measurementDate = new Date(2010, 10, 20);
    component.observationType = 'regular';
    component.doSave();
    expect(component?.observationDataRegular?.date).toEqual(component.measurementDate);
  }));

  it('should set offsetDate when offsetData exists on the dataCard', fakeAsync(() => {
    const offsetDate = new Date(2010, 6,  10);
    component.isOpen = true;
    component.activity = getActivity();
    component.observationType = 'regular';
    component.dataCard = {
      expectedDateOfDeliverySection: {
        offsetDate,
      }
    } as any;
    component.ngOnChanges({isOpen: new SimpleChange(false, true, true)});
    component.observationType = 'regular';
    component.doSave();
    expect(component?.observationDataRegular?.offsetDate).toEqual(offsetDate);
  }));

  it('should not set offsetDate when offsetData does not exists on the dataCard', fakeAsync( () => {
    component.isOpen = true;
    component.activity = getActivity();
    component.observationType = 'regular';
    component.ngOnChanges({isOpen: new SimpleChange(false, true, true)});
    component.observationType = 'regular';
    component.doSave();
    expect(component?.observationDataRegular?.offsetDate).toBeUndefined();
  }));


  it('just creates observation when in "regular" mode', fakeAsync(() => {
    component.isOpen = true;
    component.activity = getActivity();
    component.ngOnChanges({isOpen: new SimpleChange(false, true, true)});
    component.observationType = 'regular';
    const dataCardSpy = spyOn(dataCardServiceStub, 'update').and.callThrough();
    const regularObservationSpy = spyOn(observationService, 'createRegularObservations').and.callThrough();
    fixture.detectChanges();
    component.doSave();
    tick();
    expect(regularObservationSpy).toHaveBeenCalled();
  }));

  it('saves datacard and creates observations when in "ultrasound" mode', fakeAsync(() => {
    component.isOpen = true;
    component.activity = getActivity();
    component.ngOnChanges({isOpen: new SimpleChange(false, true, true)});
    const dataCardSpy = spyOn(dataCardServiceStub, 'update').and.callThrough();
    const scanObservationSpy = spyOn(observationService, 'createUltrasoundObservations').and.callThrough();
    fixture.detectChanges();
    component.observationType = 'ultrasound';
    component.doSave();
    tick();
    expect(dataCardSpy).toHaveBeenCalled();
    expect(scanObservationSpy).toHaveBeenCalled();
  }));

  it('handles that active datacard could be undefined', fakeAsync(() => {
    component = new MeasurementsModalComponent({} as any, {selectedDataCard$: of()} as any, {} as any, {} as any);
    component.ngOnInit();
    tick();
    expect(component.dataCard).toBeUndefined();
  }));

  it('should initData when obs type changes', () => {
    component.isOpen = true;
    component.ngOnChanges({observationType: new SimpleChange('regular', 'ultrasound', false)});
    expect(component.observationDataRegular?.fetuses?.length).toBe(1);
    expect(component.observationDataUltrasound?.fetuses?.length).toBe(1);
  });

  it('should handle closing when is Open changes', () => {
    component.isOpen = true;
    component.ngOnChanges({isOpen: new SimpleChange(true, false, false)});
    expect(component.modalComponent?.isModalVisible).toBeFalse();
  });

  it('should do nothing if dialog is closed while already closed', () => {
    component.isOpen = true;
    expect(component.modalComponent?.isModalVisible).toBeFalse();
    component.ngOnChanges({isOpen: new SimpleChange(false, false, false)});
    expect(component.modalComponent?.isModalVisible).toBeFalse();
  });

  describe('Edit existing activity measurements', () => {
    const activityId = '1234-abcd';
    const obsFetuses = [
      {rows: [{activityId, fetusPresentation: 'UK', gestationAge: '25+1', fetusWeightEstimate: {unit: 'g', value: 3001}, id: '123', weeksIntoPregnancy: 25}]},
      {rows: [{activityId: 'fake-id', fetusPresentation: 'H', gestationAge: '9+3', fetusWeightEstimate: {unit: 'g', value: 1020}, id: '1', weeksIntoPregnancy: 10}]}
    ];
    beforeEach(() => {
      component.activity = {
        id: activityId,
        observations: [
          {id: '1', activityReference: activityId, fetus: 'A', dateOfMeasurement: new Date(10, 8, 2019), type: 'fetus_weight_estimate', values: [{value: '2344', name: 'vægt', unit: 'g'}]},
          {id: '2', activityReference: activityId, fetus: 'A', dateOfMeasurement: new Date(10, 8, 2019), type: 'fetus_presentation', values: [{value: 'UK', name: 'Præsentation'}]}
        ],
        title: 'Vigtig Aktivitet',
        status: 'finished'
      };
    });
    it('should return an empty array when missing either activity id or fetuses in the fetus rows', () => {
      const fetusRows: ScanningObservationTableFetuses[] = [
        {rows: [], id: '123', name: 'A'},
        {rows: [], id: '222', name: 'B'},
      ];
      let relevantFetusRows = MeasurementsModalComponent.getRelevantObservationFetusRows(undefined, fetusRows);
      expect(relevantFetusRows.length).toBe(0);
      relevantFetusRows = MeasurementsModalComponent.getRelevantScanningFetusRows(undefined, fetusRows);
      expect(relevantFetusRows.length).toBe(0);
    });

    it('should find the appropriate scanning fetus rows when given an activity id', () => {
      const fetusRows: ScanningObservationTableFetuses[] = [
        {rows: [{activityId, fetusPresentation: 'UK', gestationAge: '30+1', ulWeight: {unit: 'g', value: 2344}}], id: '123', name: 'A'},
        {rows: [{activityId: 'not-this-one', fetusPresentation: 'H', gestationAge: '10+2', ulWeight: {unit: 'g', value: 506}}], id: '2', name: 'A'},
      ];
      const relevantFetusRows = MeasurementsModalComponent.getRelevantScanningFetusRows(activityId, fetusRows);
      expect(relevantFetusRows.length).toBe(1);
      expect(relevantFetusRows[0].fetusPresentation).toBe('UK');
    });

    it('should find the appropriate observation fetus rows when given an activity id', () => {
      const fetusRows: ObservationTableFetuses[] = obsFetuses;
      const relevantFetusRows = MeasurementsModalComponent.getRelevantObservationFetusRows(activityId, fetusRows);
      expect(relevantFetusRows.length).toBe(1);
      expect(relevantFetusRows[0].fetusWeightEstimate).toBe(3001);
    });

    it('should initiate observation  data with ultrasound scannings', fakeAsync(async () => {
      const expectedDvpValue = 1337;
      observationService.clinicalObservationTable$ = of({mother: {rows: [{id: '2'}]}, fetuses: [{id: '1'}]});
      observationService.ultrasoundObservationTable$ = of({mother: {rows: [{id: '2', activityId, dvp: {value: expectedDvpValue}}]}, fetuses: [{id: '1'}]});
      await component.initObservationData();
      expect(component.observationDataUltrasound?.dvp).toBe(expectedDvpValue);
    }));

    it('should initiate observation  data with regular observations',  fakeAsync(async () => {
      const expectedMothersWeight = 78;
      observationService.clinicalObservationTable$ = of({mother: {rows: [{id: '2',  activityId, weight: {value: expectedMothersWeight, unit: 'kg'}}]}, fetuses: [{id: '1'}]});
      observationService.ultrasoundObservationTable$ = of({mother: {rows: [{id: '2'}]}, fetuses: [{id: '1'}]});
      await component.initObservationData();
      expect(component.observationDataRegular?.weight).toBe(expectedMothersWeight);
    }));
  });
});
